#!/usr/bin/env csi -s
;; eldritch-world.scm
;; Created 2019-04-18
#<<DOC
## BSD License

Copyright © 2019 by Mark Damon Hughes. All Rights Reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
3. Neither the name of the copyright holder nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUDOCENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
DOC

;; Imported source, not a module, use (include-relative "eldritch-world.scm")

;;______________________________________
(define kStartRoomLoc 'town-inn-room)
(define kScoreRoomLoc 'town-inn-room)

(define kMazeRandomRooms '(town-maze2 town-maze3 town-maze4 town-maze5 town-maze6 town-maze8 town-maze9)) ;; not 1 or 7
(define kMazeUnknownExitText "Further into the alleys.")

(define (maze-exits room all-exits path-room)
	{let [(path-exit (if gDebug (car all-exits)  (random-choice all-exits)))]
		(for-each (lambda (dir)
			(if (and path-room  (= dir path-exit))
				(Room-addexit! room (make-Doorway dir path-room kMazeUnknownExitText #f))
				(Room-addexit! room (make-Doorway dir (random-choice (delete (Room-id room) kMazeRandomRooms)) kMazeUnknownExitText #f))
			)
		) all-exits)
	}
)

(define (town-inn-room-out-proc)
	(msg-plain "You can just barely fit through the window.")
	(player-moveto 'town-inn-ledge)
)

(define (town-inn-ledge-down-proc)
	(msg-warning "You plummet to the ground and shatter everything.")
	(game-over)
)

(define (town-inn-ledge-east-proc)
	(msg-warning (string-trim-both #<<END
For a moment the wind lifts you up... then gravity smashes you down.
You are barely aware of your broken body rolling down the steep street to the docks and falling off into the water, where you provide a tasty treat for unusual local fish.
END
	))
	(game-over)
)

(define (town-manor-cellar-parse cmd line-rest proc)
	(cond
		[(or (equal? cmd "put")  (equal? cmd "place")  (equal? cmd "offer"))
			{let [(room (player-room))  (item (find-player-item line-rest))]
				(cond
					[(and item (equal? (Item-name item) "Beer Stein"))
						(msg-success "You place the ~A on the altar, and the world spins around you!" (Item-name item))
						(list-delete! (Actor-items gPlayer) item)
						(list-append! (Room-items room) item)
						;; first time is worth 50 points
						(when (not (hget (Actor-flags gPlayer) 'sea-temple))
							(hput! (Actor-flags gPlayer) 'sea-temple #t)
							(inc! (Actor-score gPlayer) 50)
							(msg-plain "[Your score has gone up by 50 points.]")
						)
						(player-moveto 'sea-temple)
						#t
					]
					[item  (msg-plain "You place the ~A on the altar, and it falls to the floor." (Item-name item))
						(list-delete! (Actor-items gPlayer) item)
						(list-append! (Room-items room) item)
						#t
					]
					[else  #f] ;; message given in find-player-item
				)
			}
		]
		[else  (if proc (proc cmd line-rest) #f)]
	)
)

;;______________________________________

(define (world-init)
	(set! gRooms (hash-symbol))

	{let [(room (Room-init 'town-inn-room "Inn Room"))]
		(set! (Room-desc room) (string-trim-both #<<END
The room has barely a mattress, blanket, and lavatory. A tall, narrow window looks out east on the docks and ocean.
END
		))
		(Room-addexit! room (make-Doorway kDir_D 'town-inn "Out and down the stairs." #f))
		(Room-addexit! room (make-Doorway kDir_Out 'town-inn "" #f))
		(Room-addexit! room (make-Doorway kDir_E #f "" 'town-inn-room-out-proc))
		(add-room room)
	}

	{let [(room (Room-init 'town-inn-ledge "Ledge Outside Inn Room"))]
		(set! (Room-desc room) (string-trim-both #<<END
You have a great view of the overcast sky, the docks where nothing is happening, and a dark ocean. Far out, you see a diffuse light—maybe a ship, or on a sandbar, or under the water.
But this was clearly a mistake, you can barely hang on out here, and it's a lot further to the cobble street below than you thought.
The wind grabs at you like it wants you to jump and fly.
END
		))
		(Room-addexit! room (make-Doorway kDir_W 'town-inn-room "Climb inside." #f))
		(Room-addexit! room (make-Doorway kDir_In 'town-inn-room "" #f))
		(Room-addexit! room (make-Doorway kDir_D #f "" 'town-inn-ledge-down-proc))
		(Room-addexit! room (make-Doorway kDir_E #f "" 'town-inn-ledge-east-proc))
		(add-room room)
	}

	{let [(room (Room-init 'town-inn "Inn of the Sleeping Fish"))]
		(set! (Room-desc room) (string-trim-both #<<END
A small, dingy barroom with wobbly tables along one side, a bar along the other side.
The innkeeper, a weaselly little man named Punctilious Phipp, is not here, even though you've never seen him leave.
A dusty mounted fish hangs over the door.
END
		))
		(Room-addexit! room (make-Doorway kDir_E 'town-street "A stout wooden door opens into the street." #f))
		(Room-addexit! room (make-Doorway kDir_Out 'town-street "" #f))
		(Room-addexit! room (make-Doorway kDir_D 'town-inn-cellar "A trapdoor into a dark cellar." #f))
		(Room-addexit! room (make-Doorway kDir_U 'town-inn-room "Rickety stairs rise to the inn rooms." #f))
		(add-room room)
	}

	{let [(room (Room-init 'town-inn-cellar "Dank Dark Cellar"))]
		(set! (Room-desc room) (string-trim-both #<<END
Dim light and sound from above trickles in, reluctantly. The dirt floor blends seamlessly with the grimy walls and cobwebs in all edges.
A rack of beer barrels, smaller liquor casks, and wine bottles fill the south wall, the rest is full of trash and dark shadows.
END
		))
		(Room-addexit! room (make-Doorway kDir_U 'town-inn "Rickety stairs and trapdoor up to the Inn." #f))
		;; items
		(list-append! (Room-items room) (make-Item "Beer Stein" "The innkeeper's favorite beer stein. More of a deep chalice now you look at it, tarnished silver, embossed with wavy lines or ropes rising from the bottom." kItem_Loot #f 10 10))
		(add-room room)
	}

	{let [(room (Room-init 'town-street "Main Street by the Inn"))]
		(set! (Room-desc room) (string-trim-both #<<END
A narrow cobblestone street, with the inn looming over you.
Far back to the west, you see a high-gabled Victorian mansion, the only well-built house in sight.
END
		))
		(Room-addexit! room (make-Doorway kDir_NW 'town-maze6 "Into a maze of back alleys." #f))
		(Room-addexit! room (make-Doorway kDir_E 'town-dock "Steep descent to the docks." #f))
		(Room-addexit! room (make-Doorway kDir_S 'town-gate "The street continues to a gate." #f))
		(Room-addexit! room (make-Doorway kDir_W 'town-inn "A stout wooden door under a sign of a sleeping fish." #f))
		(Room-addexit! room (make-Doorway kDir_In 'town-inn "" #f))
		(add-room room)
	}

	{let [(room (Room-init 'town-gate "Main Street Gate"))]
		(set! (Room-desc room) (string-trim-both #<<END
The main street ends in the town's southern brick wall, and an iron gate. You can't see any way to open it from here.
END
		))
		(Room-addexit! room (make-Doorway kDir_N 'town-street "The street leads into the village." #f))
		;; outside will allow unlocking the gate
		(add-room room)
	}

	{let [(room (Room-init 'town-dock "Duquesne Docks"))]
		(set! (Room-desc room) (string-trim-both #<<END
A long, old wooden pier with a couple of planks as slips, empty of boats. The ocean laps at the pier and a few seagulls wheel and chatter overhead.
A handpainted sign over a shack proclaims "Welcome to Duquesne - harbormaster Elias Phipp".
No human voice breaks the quiet.
END
		))
		(Room-addexit! room (make-Doorway kDir_W 'town-street "Up a steep road to Main Street." #f))
		(Room-addexit! room (make-Doorway kDir_In 'town-shack "The harbormaster's shack." #f))
		(add-room room)
	}

	{let [(room (Room-init 'town-shack "Harbormaster's Shack"))]
		(set! (Room-desc room) (string-trim-both #<<END
A cramped clapboard shelter, with barely space for a chair and a filthy bucket.
The plank/desk under the window shows the outline of a logbook, etched by sunlight and dust.
END
		))
		(Room-addexit! room (make-Doorway kDir_Out 'town-dock "Get out of here." #f))
		(list-append! (Room-items room) (make-Item "Logbook" (string-trim-both #<<END
The logbook lists hundreds of entries of boats arriving and leaving over decades, mostly fishing boats.
There are only a few entries in the last few years.
The final entry records Captain Jared Whatefield's ship The Devourer returning. So where is it?
END
		) kItem_Loot #f 20 10))
		(add-room room)
	}

	;; Every text adventure should have a maze. But you only have to solve most once, that's no fun.
	;; Every room has a unique if confusing description, and all exits are visible.
	;; Predefined: 1 SE goes to 5, 5 NW goes to 1, 6 SE goes to street, 7 W goes to mansion lawn.
	;; No room connects to itself.
	;; All other paths are random (any room except 1 or 7) and edges don't have exits.
	;; The happy path room order is predefined to exist, but the exits vary at random.
	;;
	;;      +---+    +---+    +---+
	;;      | 1 |    | 2 |<---| 3 |
	;;      +---+    +---+    +---+
	;;           \_    |        ^
	;;             \   |        |
	;;              \  v        |
	;;      +---+    +---+    +---+
	;;      | 4 |    | 5 |    | 6 |
	;;      +---+    +---+    +---+
	;;        |  <        \_       \
	;;        |   \_        \       \_
	;;        v     \        >        \
	;;      +---+    +---+    +---+    town-street
	;;  ----| 7 |    | 8 |<---| 9 |
	;;      +---+    +---+    +---+

	{let [(room (Room-init 'town-maze1 "Alleyways"))]
		(set! (Room-desc room) "You are in a cul-de-sac of twisty alleyways. An empty burn barrel is here.")
		(Room-addexit! room (make-Doorway kDir_E (random-choice kMazeRandomRooms) kMazeUnknownExitText #f))
		(Room-addexit! room (make-Doorway kDir_SE 'town-maze5 kMazeUnknownExitText #f))
		(Room-addexit! room (make-Doorway kDir_S (random-choice kMazeRandomRooms) kMazeUnknownExitText #f))
		;; TODO: put an item here
		(add-room room)
	}
	{let [(room (Room-init 'town-maze2 "Alleyways"))]
		(set! (Room-desc room) "You are in a maze of twisty alleyways, all alike. Boarded-up houses surround you.")
		(maze-exits room (list kDir_E kDir_SE kDir_S kDir_SW kDir_W) 'town-maze5)
		(add-room room)
	}
	{let [(room (Room-init 'town-maze3 "Alleyways"))]
		(set! (Room-desc room) "You are in an L-shaped twisty alleyway. The backs of houses surround you.")
		(maze-exits room (list kDir_W kDir_SW kDir_S) 'town-maze2)
		(add-room room)
	}
	{let [(room (Room-init 'town-maze4 "Alleyways"))]
		(set! (Room-desc room) "You are in a tight corner alleyway of a maze. Do shadows watch you from high dark windows?")
		(maze-exits room (list kDir_N kDir_NE kDir_E kDir_SE kDir_S) 'town-maze7)
		(add-room room)
	}
	{let [(room (Room-init 'town-maze5 "Alleyways"))]
		(set! (Room-desc room) "You are deep in a maze of twisty alleyways. Empty clotheslines whip against house walls.")
		(Room-addexit! room (make-Doorway kDir_NW 'town-maze1 kMazeUnknownExitText #f))
		(maze-exits room (list kDir_N kDir_NE kDir_E kDir_SE kDir_S kDir_SW kDir_W) 'town-maze9)
		(add-room room)
	}
	{let [(room (Room-init 'town-maze6 "Alleyways"))]
		(set! (Room-desc room) "You are in a maze of twisty alleyways, all alike. The houses surrounding you are boarded up.")
		(Room-addexit! room (make-Doorway kDir_SE 'town-street "You see a way back to Main Street." #f))
		(maze-exits room (list kDir_N kDir_S kDir_SW kDir_W kDir_NW) 'town-maze3)
		(add-room room)
	}
	{let [(room (Room-init 'town-maze7 "Alleyways"))]
		(set! (Room-desc room) "You are in a different part of the alleyways. You can't see a way into any of the houses.")
		(Room-addexit! room (make-Doorway kDir_W 'town-manor-lawn "You see a break in the alleys to greenery." #f))
		(maze-exits room (list kDir_N kDir_NE kDir_E) #f)
		(add-room room)
	}
	{let [(room (Room-init 'town-maze8 "Alleyways"))]
		(set! (Room-desc room) "You are in a criss-cross of twisty alleyways. A solid brick wall is on the south side, that's different.")
		(maze-exits room (list kDir_N kDir_NE kDir_E kDir_W kDir_NW) 'town-maze4)
		(add-room room)
	}
	{let [(room (Room-init 'town-maze9 "Alleyways"))]
		(set! (Room-desc room) "You are in a back corner of twisty alleyways, all different. A couple of boarded-up houses surround you.")
		(maze-exits room (list kDir_N kDir_W kDir_NW) 'town-maze8)
		(add-room room)
	}

	{let [(room (Room-init 'town-manor-lawn "Manor Lawn"))]
		(set! (Room-desc room) (string-trim-both #<<END
You are on a nice wide lawn, grown a little wild. The bushes are deformed topiary animals. There are stone benches, but nobody around to sit on them.
Above you towers a Victorian manor house once painted perhaps a bright blue, but now a faded rain-color.
To the south is a high brick wall, and southwest is an iron fence where you can see the driveway, gate, and a nice classic black car.
END
		))
		(Room-addexit! room (make-Doorway kDir_W 'town-manor-main "Into the garden door." #f)) ;; TODO: check for key in proc
		(Room-addexit! room (make-Doorway kDir_E 'town-maze7 "Back into the alleys." #f))
		(add-room room)
	}

	{let [(room (Room-init 'town-manor-main "Manor Main Floor"))]
		(set! (Room-desc room) (string-trim-both #<<END
A glass chandelier with no power doesn't light up this wide hall at all. What you can make out is lovely, fine marble floor tiles, paintings hanging on all walls. The decorative tables seem curiously empty.
END
		))
		;; TODO: family photos and knick-knacks in car
		(Room-addexit! room (make-Doorway kDir_E 'town-manor-lawn "Out to the garden lawn." #f)) ;; TODO: check for key in proc
		(Room-addexit! room (make-Doorway kDir_W 'town-manor-drive "Out to the driveway." #f)) ;; TODO: check for key in proc
		(Room-addexit! room (make-Doorway kDir_N 'town-manor-dining "Into the dining room." #f)) ;; TODO
		(Room-addexit! room (make-Doorway kDir_S 'town-manor-den "Into the den." #f)) ;; TODO
		(Room-addexit! room (make-Doorway kDir_U 'town-manor-upper "Up a grand stairway." #f)) ;; TODO
		(Room-addexit! room (make-Doorway kDir_D 'town-manor-cellar "Through a small door downstairs." #f)) ;; TODO: check for key in proc
		(add-room room)
	}

	{let [(room (Room-init 'town-manor-cellar "Manor Cellar"))]
		(set! (Room-desc room) (string-trim-both #<<END
Once a wine cellar, the slots for bottles are now empty.
An inscribed pentagram has been painted on the floor, and in the center an altar has been erected.
The altar has a space where you could place an item.
END
		))
		;; TODO: (set! (Room-lit? room) #f)
		(Room-addexit! room (make-Doorway kDir_U 'town-manor-main "Up narrow steps." #f))
		(set! (Room-parse room) 'town-manor-cellar-parse)
		(add-room room)
	}

	{let [(room (Room-init 'sea-temple "Sea Temple"))]
		(set! (Room-desc room) (string-trim-both #<<END
Dim greenish-blue light filters in from above, where water is suspended outside an air bubble.
The structure is made of five arches of rough-hewn stone with coral and mussels growing into them, surrounding a rough, wet stone floor.
Another altar with an inscribed pentagram is in the center of the space.

This area doesn't appear to be finished, and you are trapped here until the author writes more of this world!
END
		))
		;; TODO
		(add-room room)
	}

;; TODO: 'town-manor-drive leads to 'town-gate, unlock it there
;; TODO: Four side worlds reached by putting artifact on altar in wizard's house
;; Haunted House
;; Little Hut on Lost Island
;; Whatefield's Ship
;; Sea Temple

)
