#!/usr/bin/env csi -ss
;; marklib-test.scm
;; Created 2019-03-20 14:58:54
;; Copyright © 2019 by Mark Damon Hughes. All Rights Reserved.

(import scheme)
(import (chicken file))
(import (chicken format))
(import (chicken pathname))
(import (chicken process-context))
(import test)

(cond-expand
	(compiling (declare (uses marklib)))
	(else (load (make-pathname (pathname-directory (program-name)) "marklib.scm")))
)
(import marklib)

(define (all-tests)
	(test-begin)

	(test-group "Logic"
		(test-begin)
		(test "neq?" #t (neq? 1 3))
		(test "neq?" #f (neq? 1 1))
		(test "nequal?" #t (nequal? "foo" "bar"))
		(test "nequal?" #f (nequal? "foo" "foo"))
		(test "notneg?" #t (notneg? 1))
		(test "notneg?" #f (notneg? -1))
		(test "notnull?" #t (notnull? '(a b c)))
		(test "notnull?" #f (notnull? '()))
		(test "notzero?" #f (notzero? 0))
		(test "notzero?" #t (notzero? -1))
		(test "!=" #t (!= 1 3))
		(test "!=" #f (!= 0 0))
		(test-end)
	)

	(test-group "Math"
		(test-begin)
		(test "int" 0 (int 0.0))
		(test "int" 3 (int 3.14))
		(test "int" 9 (int 100/11))
		(test "idiv" 512 (idiv 1024.9 2.1))
		(test "minmax" 5 (minmax 0 10 5))
		(test "minmax" 0 (minmax 0 10 -1))
		(test "minmax" 10 (minmax 0 10 11))
		(test "real" 3.0 (real 3))
		(test "real" 2.0 (real 10/5))
		(test "sign" -1 (sign -3.14))
		(test "sign" 0 (sign 0))
		(test "sign" 1 (sign 3.14))
		(test "sum" 55 (sum '(1 2 3 4 5 6 7 8 9 10)))
		(test-end)
	)

	(test-group "Strings"
		(test-begin)
		(test "atoi" 69 (atoi "69"))
		(test "atoi" -32 (atoi "-32.4"))
		(test "atoi" 3 (atoi "   3.14  "))
		(test "atoi" 0 (atoi "wtf"))
		(test "atoi" 32 (atoi "" 32))
		(test "ord" 65 (ord #\A))
		(test "ord" 65 (ord "A"))
		(test "ord" 0 (ord ""))
		(test "padnum" "   69" (padnum 69 5))
		(test "padnum" "3  " (padnum 3 -3))
		(test "padnum" "666" (padnum 666 1))
		(test "pad" "   hi" (pad "hi" 5))
		(test "pad" "foo  " (pad "foo" -5))
		(test "pad" "hello" (pad "hello" 3))
		(test "purify-name" "OK-name" (purify-name "OK name"))
		(test "purify-name" "Na-me-fulla-crap" (purify-name " 666  Na-me, fulla crap!"))
		(test "substr" "ello" (substr "Hello" 1 10))
		(test "substr" "He" (substr "Hello" -5 2))
		(test "substr" "lo" (substr "Hello" 3 10))
		(test "substr" "" (substr "Hello" 2 0))
		(test-end)
	)

	(test-group "Data Structures"
		(test-begin)
		(test "anycompare" -1 (anycompare 1 3))
		(test "anycompare" 1 (anycompare "foo" "bar"))
		(test "anycompare" 1 (anycompare 'this 'that))
		(test "anycompare" 0 (anycompare #\a #\a))
		(test "anycompare" 1 (anycompare #t #f))
		(test "anycompare" #f (anycompare '(1) '(2)))
		(test "anyeq?" #t (anyeq? 69 69))
		(test "anyless?" #t (anyless? 42 69))
		(test "anymore?" #t (anymore? 69 42))
		(test "anylen" 3 (anylen '(1 2 3)))
		(test "anylen" 5 (anylen "Hello"))
		(test "anylen" 3 (anylen #(1 2 3)))
		(test "anylen" 3 (anylen (hash-symbol '(a 1) '(b 2) '(c 3))))
		(test "anylen" #f (anylen 69))
		(test "anyitem" 2 (anyitem '(1 2 3) 1))
		(test "anyitem" #\e (anyitem "Hello" 1))
		(test "anyitem" 2 (anyitem #(1 2 3) 1))
		(test "anyitem" 2 (anyitem (hash-symbol '(a 1) '(b 2) '(c 3)) 'b))
		(test "anyitem" #f (anyitem 69 1))
		{let [(animals '((a aardvark) (b bull 2) (c cow 3) (d . dog) (z)) )]
			(test "alget" #f (alget '(junk) 'x)) ;; not even an alist
			(test "alget" #f (alget animals 'y))
			(test "alget" '() (alget animals 'z))
			(test "alget" 'aardvark (alget animals 'a))
			(test "alget" '(bull 2) (alget animals 'b))
			(test "alget" 'dog (alget animals 'd))
		}

		(test-group "hash"
			{let [(ht (hash-symbol '(a 1) '(b 2) '(c 3)))]
				(test "anylen" 3 (anylen ht))
				(test "anyitem" 1 (anyitem ht 'a))
				(hadd! ht 'd 1)
				(test "anyitem" 1 (anyitem ht 'd))
				(hadd! ht 'd 2)
				(test "anyitem" 3 (anyitem ht 'd))
				(hdel! ht 'd)
				(test "anyitem" #f (anyitem ht 'd))
				(test "hget" 1 (hget ht 'a 69))
				(test "hget" 69 (hget ht 'q 69))
				(hput! ht 'r 69)
				(test "hget" 69 (hget ht 'r 0))
				(test "hselect" 'r (hselect ht (lambda (x) (>= x 69))))
				(test "hstring" "a: 1\nb: 2\nc: 3\nr: 69\n" (hstring ht))
				(test "hstring optional" "<a=1><b=2><c=3><r=69>" (hstring ht "<" "=" ">"))
			}
		)

		(test-group "list-append"
			{let [(ls '())]
				(list-append! ls 1)
				(test "list-append!" ls '(1))
				(list-append! ls '(2 3))
				(test "list-append!" ls '(1 (2 3)))
				(list-delete! ls 1)
				(test "list-delete!" ls '((2 3)))
			}
		)

		(test-group "maybe"
			(test "maybe-car" 1 (maybe-car '(1 2 3)) )
			(test "maybe-car" #f (maybe-car '()) )
			(test "maybe-car" #f (maybe-car "stuff") )
			(test "maybe-cdr" '(2 3) (maybe-cdr '(1 2 3)) )
			(test "maybe-cdr" '() (maybe-cdr '()) )
			(test "maybe-cdr" '() (maybe-cdr "stuff") )
		)

		(test-group "Stack"
			{let [(st (make-Stack))]
				(st 'push! 1)
				(st 'push! 2)
				(test "stack" 2 (st 'length))
				(test "stack" (list 2 1) (st 'list))
				(test "stack" 2 (st 'peek))
				(test "stack" 2 (st 'pop!))
				(test "stack" #f (st 'empty?))
				(test "stack" 1 (st 'peek))
				(test "stack" 1 (st 'pop!))
				(test "stack" #t (st 'empty?))
			}
		)

		{let [(v (vector 1 2 3 4 5 6))]
			(test "vector" #(1 2 3 4 5 6) v)
			(test "vector" 6 (vector-length v))
			(test "vget" 1 (vget v 0))
			(test "vget" #f (vget v 10))
			(test "vindex" 1 (vindex v 2))
			(test "vindex" #f (vindex v 23))
			(test "vset" #(1 69 3 4 5 6) (begin (vset! v 1 69) v))
		}

		(test-end)
	)

	(test-group "IO"
		(test-begin)
		;; FIXME: only testing happy path
		(test-group "data-file"
			{let [(tmpfile (create-temporary-file ".dat"))]
				(test "write-data-file" #t (write-data-file tmpfile '(1 2 3)))
				(test "read-data-file" '(1 2 3) (read-data-file tmpfile))
			}
		)

		(test-group "text-file"
			{let [(tmpfile (create-temporary-file ".txt"))]
				(test "write-text-file" #t (write-text-file tmpfile "Hello, world!\n"))
				(test "read-text-file" "Hello, world!\n" (read-text-file tmpfile))
			}
		)
		(test-end)
	)

	(test-group "Dice"
		(test-begin)
		{let [(seed (intlist->string '(1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16)))]
			(dice-init seed)
			(test '(121 97 537) (list (die 1000) (die 1000) (die 1000)))
			(dice-init seed)
			(test '(121 97 537) (list (die 1000) (die 1000) (die 1000)))
		}
		(test-end)
	)

	;; TODO: test Text UI, Geometry

	(test-group "Point"
		(test-begin)
		{let [(pt1 (make-Point 0 0))  (pt2 (make-Point 0 0))]
			(test "Point string" "#,(Point 0 0)" (sprintf "~A" pt1))
			(Point-x-add! pt2 3)
			(Point-y-add! pt2 4)
			(test "Point-x" 3 (Point-x pt2))
			(test "Point-y" 4 (Point-y pt2))
			(Point-xy-set! pt2 5 10)
			(test "Point-copy" (make-Point 5 10)  (Point-copy pt2))
			(test "Point-scale-xy" (make-Point 10 -10)  (Point-scale-xy pt2 2 -1))
			(test "Point-translate" (make-Point 6 12)  (Point-translate pt2 (make-Point 1 2)))
			(test "Point-line-to-point" '(#,(Point 0 0) #,(Point 0 1) #,(Point 1 2) #,(Point 1 3) #,(Point 2 4) #,(Point 2 5) #,(Point 3 6) #,(Point 3 7) #,(Point 4 8) #,(Point 4 9) #,(Point 5 10))  (Point-line-to-point pt1 pt2) )
		}
		(test-end)
	)

	(test-end)
	(test-exit)
)

(define (main argv)
	(all-tests)
)
